% Inverse problem for elastic substrate method
% Ulrich Schwarz, MPI Colloids and Interfaces, Germany,
% http://www.mpikg-golm.mpg.de/th/people/schwarz/
% written May 2000, modified Feb 2003 

%inverse.m solves inverse problem
%INPUT:   newlines.dat contains displacements as produced by data
%         focals.dat contains focal positions as produced by data
%OUTPUT:  forces.dat contains forces as starting from focal positions

% add directory with the Hansen routines for inverse problems to the path
% please see his README-file for more details
path(path,'Hansen')

% standard deviation for noise in displacement
% from this we can calculate an estimate for the regularization parameter lambda
sigma = 1;

% load displacement vectors 
lines = load('newlines.dat');

% displacements is stored both as one large vector u and as a list of coordinates
u = m2v(lines(:,3:4)-lines(:,1:2));
displ = v2m(u);

% load positions of focals
load('focals.dat');

% for control purposes, we use artificial force patterns
% then we can also load the original force
load('forces_orig.dat');

% number of degrees of freedom
DF = 2*(size(lines,1)-size(focals,1));
fprintf('%d degrees of freedom\n',DF)
% estimate for residual norm and its confidence interval
rho_opt = sqrt(DF)*sigma;
rho_opt_plus = (sqrt(DF)+(2*DF)^0.25)*sigma;
rho_opt_minus = (sqrt(DF)-(2*DF)^0.25)*sigma;
fprintf('sigma = %f\n',sigma)
fprintf('Optimal rho = %6.0f\n',rho_opt)

% plot data
data_plot(lines,focals)
wait

% construct Greens function
fprintf('Calculating G ...\n')

% build Greens function between the two sets of positions
for i = 1:size(lines,1)
   for j = 1:size(focals,1)
      G(2*i-1:2*i,2*j-1:2*j) = greens(lines(i,1:2)-focals(j,1:2));
   end
end

% singular value decomposition for G
[U,s,V] = csvd(G);

% show L-curve
lambda_v = logspace(-3,1,30);
ls = size(lambda_v,2);
[F_v,rho_v,eta_v]   = tikhonov(U,s,V,u,lambda_v);
semilogx(lambda_v,rho_v,'b'), hold on
semilogx(lambda_v,rho_opt*ones(size(lambda_v)),'r-')
semilogx(lambda_v,rho_opt_plus*ones(size(lambda_v)),'r:')
semilogx(lambda_v,rho_opt_minus*ones(size(lambda_v)),'r:')
title('Choice of regularization parameter')
xlabel('l','fontname','symbol')
ylabel('residual norm || A x - b ||_2')
wait

while 1
    
    % input regularization parameter
    lambda = input('Regularization parameter ? (0 to exit) ');
    if lambda == 0
        break
    end
    
    fprintf('Used regularization parameter = %f\n',lambda)
    
    % solve inverse problem 
    fprintf('Solving inverse problem ...\n')
    [F,rho,eta]   = tikhonov(U,s,V,u,lambda);
    
    % forces as matrix
    forces = v2m(F);
    % save forces
    save 'forces.dat' forces -ascii
    fprintf('Forces saved in forces.dat\n')
    
    % plot everything
    %data_plot(lines,focals,forces)
    % do not plot lines
    aplot(focals,forces,'-k'), hold on
    aplot(focals,forces_orig,'-r')
    splot(focals,'ks')
    axis equal
    
    wait
    
end


